"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ItemNode = void 0;
const NodeContext_1 = require("./NodeContext");
const NODE_MAX_DEPTH = 3;
class ItemNode {
    value;
    label;
    context;
    constructor(value, label, children = []) {
        this.value = value;
        this.label = label;
        this.context = new NodeContext_1.NodeContext(this);
        for (const child of children) {
            this.append(child);
        }
    }
    static from(item) {
        return new ItemNode(item.value, item.label, (item.children ?? []).map((itemNodeLike) => ItemNode.from(itemNodeLike)));
    }
    get parent() {
        return this.context.parent?.node;
    }
    get children() {
        return this.context.children.map((child) => child.node);
    }
    append(that) {
        if (this.#getDepth() >= NODE_MAX_DEPTH) {
            throw new Error(`Cannot append to a node deeper than ${NODE_MAX_DEPTH}`);
        }
        this.context.append(that.context);
    }
    getNext() {
        if (!this.parent) {
            return;
        }
        const index = this.#getIndex();
        if (index === -1) {
            return;
        }
        return this.parent.children[index + 1];
    }
    getPrev() {
        if (!this.parent) {
            return;
        }
        const index = this.#getIndex();
        if (index === -1) {
            return;
        }
        return this.parent.children[index - 1];
    }
    getFirstChild() {
        return this.children.at(0);
    }
    getLastChild() {
        return this.children.at(-1);
    }
    getAncestors() {
        const ancestors = [];
        let current = this.parent;
        while (current instanceof ItemNode) {
            ancestors.unshift(current);
            current = current.parent;
        }
        return ancestors;
    }
    getSiblings() {
        if (!this.parent) {
            return [];
        }
        return this.parent.children;
    }
    findByValue(value) {
        if (this.value === value) {
            return this;
        }
        for (const child of this.children) {
            const found = child.findByValue(value);
            if (found) {
                return found;
            }
        }
        return;
    }
    #getIndex() {
        if (!this.parent) {
            return -1;
        }
        return this.parent.children.indexOf(this);
    }
    #getDepth() {
        return this.getAncestors().length + 1;
    }
}
exports.ItemNode = ItemNode;
//# sourceMappingURL=ItemNode.js.map