"use strict";
'use client';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelessDialog = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const react_draggable_1 = __importDefault(require("react-draggable"));
const tailwind_variants_1 = require("tailwind-variants");
const useHandleEscape_1 = require("../../../hooks/useHandleEscape");
const intl_1 = require("../../../intl");
const Base_1 = require("../../Base");
const Button_1 = require("../../Button");
const Heading_1 = require("../../Heading");
const Icon_1 = require("../../Icon");
const DialogBody_1 = require("../DialogBody");
const DialogOverlap_1 = require("../DialogOverlap");
const useDialogPortal_1 = require("../useDialogPortal");
const classNameGenerator = (0, tailwind_variants_1.tv)({
    slots: {
        overlap: 'shr-inset-[unset]',
        wrapper: 'smarthr-ui-ModelessDialog shr-fixed shr-flex shr-flex-col',
        headerEl: 'smarthr-ui-ModelessDialog-header shr-border-b-shorthand shr-relative shr-flex shr-cursor-move shr-items-center shr-rounded-tl-l shr-rounded-tr-l shr-pe-1 shr-ps-1.5 hover:shr-bg-white-darken',
        dialogHandler: [
            'smarthr-ui-ModelessDialog-handle shr-absolute shr-inset-x-0 shr-bottom-0 shr-top-[2px] shr-m-auto shr-flex shr-justify-center shr-rounded-tl-s shr-rounded-tr-s shr-text-grey shr-transition-colors shr-duration-100 shr-ease-in-out',
            'focus-visible:shr-bg-white-darken focus-visible:shr-shadow-outline focus-visible:shr-outline-none',
        ],
        titleEl: [
            'shr-my-1 shr-me-1',
            /* DialogHandlerの上に出すためにスタッキングコンテキストを生成 */
            '[.smarthr-ui-ModelessDialog-handle:focus-visible_+_&]:shr-relative',
        ],
        closeButtonLayout: [
            'shr-relative' /* DialogHandlerの上に出すためにスタッキングコンテキストを生成 */,
            'shr-ml-auto shr-shrink-0',
        ],
        footerEl: 'smarthr-ui-ModelessDialog-footer shr-border-t-shorthand',
    },
    variants: {
        resizable: {
            true: {
                wrapper: 'shr-resize shr-overflow-auto',
            },
            false: {},
        },
    },
});
const ModelessDialog = ({ title, children, contentBgColor, contentPadding, footer, isOpen, onPressEscape, resizable = false, width, height, top, left, right, bottom, portalParent, className, decorators, id, onClickClose, ...props }) => {
    const labelId = (0, react_1.useId)();
    const lastFocusElementRef = (0, react_1.useRef)(null);
    const { createPortal } = (0, useDialogPortal_1.useDialogPortal)(portalParent, id);
    const { localize } = (0, intl_1.useIntl)();
    const classNames = (0, react_1.useMemo)(() => {
        const { overlap, wrapper, headerEl, titleEl, dialogHandler, closeButtonLayout, footerEl } = classNameGenerator();
        return {
            overlap: overlap({ className }),
            wrapper: wrapper({ resizable }),
            header: headerEl(),
            title: titleEl(),
            dialogHandler: dialogHandler(),
            closeButtonLayout: closeButtonLayout(),
            footer: footerEl(),
        };
    }, [className, resizable]);
    const wrapperRef = (0, react_1.useRef)(null);
    const focusTargetRef = (0, react_1.useRef)(null);
    const [wrapperPosition, setWrapperPosition] = (0, react_1.useState)(undefined);
    const [centering, setCentering] = (0, react_1.useState)({});
    const [position, setPosition] = (0, react_1.useState)({
        x: 0,
        y: 0,
    });
    const [draggableBounds, setDraggableBounds] = (0, react_1.useState)();
    const decoratorDefaultTexts = (0, react_1.useMemo)(() => ({
        closeButtonIconAlt: localize({
            id: 'smarthr-ui/ModelessDialog/closeButtonIconAlt',
            defaultText: '閉じる',
        }),
        dialogHandlerAriaLabel: localize({
            id: 'smarthr-ui/ModelessDialog/dialogHandlerAriaLabel',
            defaultText: 'ダイアログの位置',
        }),
        dialogHandlerAriaValuetext: wrapperPosition
            ? localize({
                id: 'smarthr-ui/ModelessDialog/dialogHandlerAriaValuetext',
                defaultText: '上から{top}px、左から{left}px',
            }, {
                top: Math.trunc(wrapperPosition.top).toString(),
                left: Math.trunc(wrapperPosition.left).toString(),
            })
            : '',
    }), [localize, wrapperPosition]);
    const decorated = (0, react_1.useMemo)(() => {
        if (!decorators) {
            return {
                dialogHandlerAriaLabel: decoratorDefaultTexts.dialogHandlerAriaLabel,
                closeButtonIconAlt: decoratorDefaultTexts.closeButtonIconAlt,
                dialogHandlerAriaValuetext: decoratorDefaultTexts.dialogHandlerAriaValuetext,
            };
        }
        return {
            dialogHandlerAriaLabel: decorators.dialogHandlerAriaLabel?.(decoratorDefaultTexts.dialogHandlerAriaLabel) ||
                decoratorDefaultTexts.dialogHandlerAriaLabel,
            closeButtonIconAlt: decorators.closeButtonIconAlt?.(decoratorDefaultTexts.closeButtonIconAlt) ||
                decoratorDefaultTexts.closeButtonIconAlt,
            dialogHandlerAriaValuetext: decorators.dialogHandlerAriaValuetext?.(decoratorDefaultTexts.dialogHandlerAriaValuetext, wrapperPosition),
        };
    }, [decorators, decoratorDefaultTexts, wrapperPosition]);
    const positionStyle = (0, react_1.useMemo)(() => ({
        top: centering.top ?? top,
        left: centering.left ?? left,
        right,
        bottom,
        width,
        height,
    }), [centering, top, left, right, bottom, width, height]);
    const handleArrowKey = (0, react_1.useCallback)((e) => {
        if (!isOpen || document.activeElement !== e.currentTarget) {
            return;
        }
        const movingDistance = 20;
        switch (e.key) {
            case 'ArrowUp':
                setPosition((prev) => ({
                    x: prev.x,
                    y: prev.y - movingDistance,
                }));
                e.preventDefault();
                break;
            case 'ArrowDown':
                setPosition((prev) => ({
                    x: prev.x,
                    y: prev.y + movingDistance,
                }));
                e.preventDefault();
                break;
            case 'ArrowLeft':
                setPosition((prev) => ({
                    x: prev.x - movingDistance,
                    y: prev.y,
                }));
                e.preventDefault();
                break;
            case 'ArrowRight':
                setPosition((prev) => ({
                    x: prev.x + movingDistance,
                    y: prev.y,
                }));
                e.preventDefault();
                break;
        }
    }, [isOpen]);
    (0, react_1.useEffect)(() => {
        if (wrapperRef.current instanceof Element) {
            setWrapperPosition(wrapperRef.current.getBoundingClientRect());
        }
    }, [position]);
    (0, react_1.useEffect)(() => {
        // 中央寄せの座標計算を行う
        if (!wrapperRef.current || !isOpen) {
            return;
        }
        const isXCenter = left === undefined && right === undefined;
        const isYCenter = top === undefined && bottom === undefined;
        if (isXCenter || isYCenter) {
            const rect = wrapperRef.current.getBoundingClientRect();
            setCentering({
                top: isYCenter ? window.innerHeight / 2 - rect.height / 2 : undefined,
                left: isXCenter ? window.innerWidth / 2 - rect.width / 2 : undefined,
            });
        }
    }, [bottom, isOpen, left, right, top]);
    (0, react_1.useEffect)(() => {
        if (!isOpen)
            return;
        if (centering.top) {
            setDraggableBounds({ top: centering.top * -1 });
            return;
        }
        if (wrapperRef.current) {
            const rect = wrapperRef.current.getBoundingClientRect();
            setDraggableBounds({ top: rect.top * -1 });
        }
    }, [isOpen, centering.top]);
    (0, react_1.useEffect)(() => {
        if (isOpen) {
            setPosition({ x: 0, y: 0 });
            focusTargetRef.current?.focus();
        }
    }, [isOpen]);
    const actualOnClickClose = (0, react_1.useCallback)((e) => {
        lastFocusElementRef.current?.focus();
        onClickClose?.(e);
    }, [onClickClose]);
    const actualOnPressEscape = (0, react_1.useMemo)(() => onPressEscape
        ? () => {
            lastFocusElementRef.current?.focus();
            onPressEscape();
        }
        : undefined, [onPressEscape]);
    (0, useHandleEscape_1.useHandleEscape)((0, react_1.useMemo)(() => (actualOnPressEscape && isOpen ? actualOnPressEscape : undefined), [isOpen, actualOnPressEscape]));
    (0, react_1.useEffect)(() => {
        const focusHandler = (e) => {
            // e.target(現在フォーカスがあたっている要素)がModeless dialog外の要素であれば、lastFocusElementRefに代入する
            if (e.target instanceof HTMLElement && !wrapperRef?.current?.contains(e.target)) {
                lastFocusElementRef.current = e.target;
            }
        };
        document.addEventListener('focus', focusHandler, true);
        return () => document.removeEventListener('focus', focusHandler, true);
    }, []);
    const onDragStart = (0, react_1.useCallback)((_, data) => setPosition(data), []);
    const onDrag = (0, react_1.useCallback)((_, data) => {
        setPosition((prev) => ({
            x: prev.x + data.deltaX,
            y: prev.y + data.deltaY,
        }));
    }, []);
    return createPortal((0, jsx_runtime_1.jsx)(DialogOverlap_1.DialogOverlap, { isOpen: isOpen, className: classNames.overlap, as: "section", children: (0, jsx_runtime_1.jsx)(react_draggable_1.default, { handle: ".smarthr-ui-ModelessDialog-handle", onStart: onDragStart, onDrag: onDrag, position: position, bounds: draggableBounds, nodeRef: wrapperRef, children: (0, jsx_runtime_1.jsxs)(Base_1.Base, { ...props, ref: wrapperRef, role: "dialog", "aria-labelledby": labelId, radius: "m", layer: 3, overflow: "auto", className: classNames.wrapper, style: positionStyle, children: [(0, jsx_runtime_1.jsx)("div", { tabIndex: -1, ref: focusTargetRef }), (0, jsx_runtime_1.jsxs)("div", { className: classNames.header, children: [(0, jsx_runtime_1.jsx)(Handler, { "aria-label": decorated.dialogHandlerAriaLabel, "aria-valuetext": decorated.dialogHandlerAriaValuetext, onArrowKeyDown: handleArrowKey, className: classNames.dialogHandler }), (0, jsx_runtime_1.jsx)("div", { id: labelId, className: classNames.title, children: (0, jsx_runtime_1.jsx)(Heading_1.Heading, { children: title }) }), (0, jsx_runtime_1.jsx)(CloseButton, { onClick: actualOnClickClose, className: classNames.closeButtonLayout, iconAlt: decorated.closeButtonIconAlt })] }), (0, jsx_runtime_1.jsx)(DialogBody_1.DialogBody, { contentBgColor: contentBgColor, contentPadding: contentPadding, className: "smarthr-ui-ModelessDialog-content shr-overscroll-contain", children: children }), footer && (0, jsx_runtime_1.jsx)("div", { className: classNames.footer, children: footer })] }) }) }));
};
exports.ModelessDialog = ModelessDialog;
const Handler = (0, react_1.memo)(({ onArrowKeyDown, ...rest }) => (
// eslint-disable-next-line jsx-a11y/role-has-required-aria-props
(0, jsx_runtime_1.jsx)("div", { ...rest, tabIndex: 0, role: "slider", onKeyDown: onArrowKeyDown, children: (0, jsx_runtime_1.jsx)(Icon_1.FaGripIcon, {}) })));
const CloseButton = (0, react_1.memo)(({ onClick, iconAlt, className }) => ((0, jsx_runtime_1.jsx)("div", { className: className, children: (0, jsx_runtime_1.jsx)(Button_1.Button, { type: "button", size: "s", onClick: onClick, className: "smarthr-ui-ModelessDialog-closeButton", children: (0, jsx_runtime_1.jsx)(Icon_1.FaXmarkIcon, { alt: iconAlt }) }) })));
//# sourceMappingURL=ModelessDialog.js.map