"use strict";
'use client';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Textarea = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const tailwind_variants_1 = require("tailwind-variants");
const useDecorators_1 = require("../../hooks/useDecorators");
const debounce_1 = require("../../libs/debounce");
const themes_1 = require("../../themes");
const createFontSize_1 = require("../../themes/createFontSize");
const VisuallyHiddenText_1 = require("../VisuallyHiddenText");
const getStringLength = (value) => {
    const formattedValue = typeof value === 'number' || typeof value === 'string'
        ? `${value}`
        : Array.isArray(value)
            ? value.join(',')
            : '';
    // https://developer.mozilla.org/ja/docs/Web/JavaScript/Reference/Global_Objects/String/charCodeAt
    const surrogatePairs = /[\uD800-\uDBFF][\uDC00-\uDFFF]/g;
    return formattedValue.length - (formattedValue.match(surrogatePairs) || []).length;
};
const DECORATOR_DEFAULT_TEXTS = {
    beforeMaxLettersCount: 'あと',
    afterMaxLettersCount: '文字',
    afterMaxLettersCountExceeded: 'オーバー',
    beforeScreenReaderMaxLettersDescription: '最大',
    afterScreenReaderMaxLettersDescription: '文字入力できます',
};
const classNameGenerator = (0, tailwind_variants_1.tv)({
    slots: {
        textareaEl: [
            'smarthr-ui-Textarea-textarea',
            'shr-border-shorthand shr-my-[unset] shr-box-border shr-rounded-m shr-bg-white shr-p-0.5 shr-text-base shr-leading-normal shr-text-black shr-opacity-100',
            'contrast-more:shr-border-high-contrast',
            'placeholder:shr-text-grey',
            'focus-visible:shr-focus-indicator',
            'disabled:shr-pointer-events-none disabled:shr-bg-column disabled:shr-text-disabled disabled:placeholder:shr-text-disabled',
            'aria-[invalid]:shr-border-danger',
        ],
        counter: 'smarthr-ui-Textarea-counter shr-block shr-text-sm shr-text-black',
    },
    variants: {
        error: {
            true: {
                counter: 'shr-text-danger',
            },
        },
    },
    defaultVariants: {
        error: false,
    },
});
const calculateIdealRows = (element, maxRows = Number.MAX_SAFE_INTEGER) => {
    if (!element) {
        return 0;
    }
    // 現在の入力値に応じた行数
    const currentInputValueRows = Math.floor(element.scrollHeight / (createFontSize_1.defaultHtmlFontSize * Number(themes_1.lineHeight.normal)));
    return currentInputValueRows < maxRows ? currentInputValueRows : maxRows;
};
exports.Textarea = (0, react_1.forwardRef)(({ autoFocus, maxLetters, width, className, autoResize = false, maxRows = Infinity, rows = 2, onInput, decorators, error, onChange, value, ...props }, ref) => {
    const maxLettersId = (0, react_1.useId)();
    const maxLettersNoticeId = `${maxLettersId}-notice`;
    const actualMaxLettersId = maxLetters ? maxLettersId : undefined;
    const textareaRef = (0, react_1.useRef)(null);
    const currentValue = props.defaultValue || value;
    const [interimRows, setInterimRows] = (0, react_1.useState)(rows);
    const [count, setCount] = (0, react_1.useState)(currentValue ? getStringLength(currentValue) : 0);
    const [srCounterMessage, setSrCounterMessage] = (0, react_1.useState)('');
    const decorated = (0, useDecorators_1.useDecorators)(DECORATOR_DEFAULT_TEXTS, decorators);
    const getCounterMessage = (0, react_1.useCallback)((counterValue) => {
        if (maxLetters === undefined)
            return;
        if (counterValue > maxLetters) {
            // {count}文字オーバー
            return ((0, jsx_runtime_1.jsxs)(jsx_runtime_1.Fragment, { children: [counterValue - maxLetters, decorated.afterMaxLettersCount, decorated.afterMaxLettersCountExceeded] }));
        }
        // あと{count}文字
        return ((0, jsx_runtime_1.jsxs)(jsx_runtime_1.Fragment, { children: [decorated.beforeMaxLettersCount, maxLetters - counterValue, decorated.afterMaxLettersCount] }));
    }, [
        maxLetters,
        decorated.afterMaxLettersCountExceeded,
        decorated.afterMaxLettersCount,
        decorated.beforeMaxLettersCount,
    ]);
    const counterVisualMessage = (0, react_1.useMemo)(() => getCounterMessage(count), [count, getCounterMessage]);
    (0, react_1.useImperativeHandle)(ref, () => textareaRef.current);
    const debouncedUpdateCount = (0, react_1.useMemo)(() => maxLetters
        ? (0, debounce_1.debounce)((newValue) => {
            (0, react_1.startTransition)(() => {
                setCount(getStringLength(newValue));
            });
        }, 200)
        : undefined, [maxLetters]);
    // countが連続で更新されると、スクリーンリーダーが古い値を読み上げてしまうため、メッセージの更新を遅延しています
    const debouncedUpdateSrCounterMessage = (0, react_1.useMemo)(() => maxLetters
        ? (0, debounce_1.debounce)((newValue) => {
            (0, react_1.startTransition)(() => {
                const counterText = getCounterMessage(getStringLength(newValue));
                if (counterText) {
                    setSrCounterMessage(counterText);
                }
            });
        }, 1000)
        : undefined, [maxLetters, getCounterMessage]);
    const handleChange = (0, react_1.useCallback)((e) => {
        const newValue = e.target.value;
        debouncedUpdateCount?.(newValue);
        debouncedUpdateSrCounterMessage?.(newValue);
        onChange?.(e);
    }, [onChange, debouncedUpdateCount, debouncedUpdateSrCounterMessage]);
    // autoFocus時に、フォーカスを当てる
    (0, react_1.useEffect)(() => {
        if (autoFocus && textareaRef && textareaRef.current) {
            textareaRef.current.focus();
        }
    }, [autoFocus]);
    // autoResize時に、初期値での高さを指定
    (0, react_1.useEffect)(() => {
        if (autoResize && textareaRef.current) {
            setInterimRows(calculateIdealRows(textareaRef.current, maxRows));
        }
    }, [setInterimRows, maxRows, autoResize]);
    // value 変更時にもカウントを更新する
    (0, react_1.useEffect)(() => {
        if (value && maxLetters) {
            debouncedUpdateCount?.(value);
            debouncedUpdateSrCounterMessage?.(value);
        }
    }, [maxLetters, debouncedUpdateCount, debouncedUpdateSrCounterMessage, value]);
    const handleInput = (0, react_1.useCallback)((e) => {
        // rowsを初期化 TextareaのscrollHeightが文字列削除時に変更されないため
        e.target.rows = rows;
        if (autoResize) {
            const currentRows = calculateIdealRows(e.target, maxRows);
            // rowsを直接反映 Textareaのrows propsが状態を変更しても反映されないため
            e.target.rows = currentRows;
            setInterimRows(currentRows);
        }
        onInput?.(e);
    }, [autoResize, maxRows, onInput, rows]);
    const textareaStyle = (0, react_1.useMemo)(() => ({ width: typeof width === 'number' ? `${width}px` : width }), [width]);
    const countError = maxLetters && count > maxLetters;
    const classNames = (0, react_1.useMemo)(() => {
        const { textareaEl, counter } = classNameGenerator();
        return {
            textarea: textareaEl({ className }),
            counter: counter({ error: !!countError }),
        };
    }, [countError, className]);
    const body = ((0, jsx_runtime_1.jsx)("textarea", { ...props, ...(maxLetters && { 'aria-describedby': `${maxLettersNoticeId} ${actualMaxLettersId}` }), "data-smarthr-ui-input": "true", value: value, onChange: handleChange, ref: textareaRef, "aria-invalid": error || countError || undefined, rows: interimRows, onInput: handleInput, className: classNames.textarea, style: textareaStyle }));
    return maxLetters ? ((0, jsx_runtime_1.jsxs)("span", { children: [body, (0, jsx_runtime_1.jsxs)(VisuallyHiddenText_1.VisuallyHiddenText, { id: maxLettersNoticeId, children: [decorated.beforeScreenReaderMaxLettersDescription, maxLetters, decorated.afterScreenReaderMaxLettersDescription] }), (0, jsx_runtime_1.jsx)(VisuallyHiddenText_1.VisuallyHiddenText, { "aria-live": "polite", children: srCounterMessage }), (0, jsx_runtime_1.jsx)("span", { id: actualMaxLettersId, "aria-hidden": true, className: classNames.counter, children: counterVisualMessage })] })) : (body);
});
//# sourceMappingURL=Textarea.js.map