"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TooltipPortal = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const tailwind_variants_1 = require("tailwind-variants");
const debounce_1 = require("../../libs/debounce");
const themes_1 = require("../../themes");
const Balloon_1 = require("../Balloon");
const classNameGenerator = (0, tailwind_variants_1.tv)({
    slots: {
        container: 'smarthr-ui-Tooltip-popup shr-absolute shr-z-overlap aria-hidden:shr-hidden',
        balloon: 'shr-max-w-full [&&&]:shr-whitespace-normal',
        balloonText: 'shr-m-0 shr-px-1 shr-py-0.5',
    },
});
const OUTER_MARGIN = 10;
const SPACING = 5;
const TooltipPortal = ({ message, isVisible, parentRect, isIcon }) => {
    const portalRef = (0, react_1.useRef)(null);
    const [style, setStyle] = (0, react_1.useState)({});
    const [actualHorizontal, setActualHorizontal] = (0, react_1.useState)('center');
    const [actualVertical, setActualVertical] = (0, react_1.useState)('bottom');
    (0, react_1.useEffect)(() => {
        if (!portalRef.current || !parentRect) {
            return;
        }
        const portal = portalRef.current;
        const action = () => {
            const vertical = calculateVertical(portal.offsetHeight, parentRect);
            const horizontal = calculateHorizontal(portal.offsetWidth, parentRect);
            setStyle({
                insetBlockStart: vertical.insetBlockStart,
                insetInlineStart: horizontal.insetInlineStart,
                insetInlineEnd: horizontal.insetInlineEnd,
                maxWidth: horizontal.maxWidth,
                maxHeight: vertical.maxHeight,
            });
            setActualVertical(vertical.alignment);
            setActualHorizontal(horizontal.alignment);
        };
        const debouncedAction = (0, debounce_1.debounce)(action, 100);
        action();
        window.addEventListener('resize', debouncedAction);
        return () => {
            window.removeEventListener('resize', debouncedAction);
        };
    }, [parentRect]);
    const classNames = (0, react_1.useMemo)(() => {
        const { container, balloon, balloonText } = classNameGenerator();
        return {
            container: container(),
            balloon: balloon(),
            balloonText: balloonText(),
        };
    }, []);
    return ((0, jsx_runtime_1.jsx)("div", { ref: portalRef, role: "tooltip", "aria-hidden": !isVisible, className: classNames.container, style: style, children: (0, jsx_runtime_1.jsx)(Balloon_1.Balloon, { horizontal: actualHorizontal, vertical: actualVertical, triggerIcon: isIcon, className: classNames.balloon, children: (0, jsx_runtime_1.jsx)("div", { className: classNames.balloonText, children: message }) }) }));
};
exports.TooltipPortal = TooltipPortal;
const calculateVertical = (portalHeight, parentRect) => {
    // トリガの上側の領域に収まる場合
    if (parentRect.top - portalHeight >= 0) {
        return {
            insetBlockStart: `${scrollY + parentRect.top - portalHeight - SPACING}px`,
            maxHeight: undefined,
            alignment: 'bottom',
        };
    }
    // トリガの下側の領域に収まる場合
    if (parentRect.bottom + portalHeight <= innerHeight) {
        return {
            insetBlockStart: `${scrollY + parentRect.bottom + SPACING}px`,
            maxHeight: undefined,
            alignment: 'top',
        };
    }
    const triggerHeight = parentRect.bottom - parentRect.top;
    // 上側の領域のほうが広い場合
    if (parentRect.top + triggerHeight / 2 >= innerHeight / 2) {
        return {
            insetBlockStart: `${scrollY + OUTER_MARGIN - SPACING}px`,
            maxHeight: `${parentRect.top - OUTER_MARGIN}px`,
            alignment: 'bottom',
        };
    }
    // 下側の領域のほうが広い場合
    return {
        insetBlockStart: `${scrollY + parentRect.bottom + SPACING}px`,
        maxHeight: `${innerHeight - parentRect.bottom - OUTER_MARGIN}px`,
        alignment: 'top',
    };
};
const calculateHorizontal = (portalWidth, parentRect) => {
    const triggerAlignCenter = parentRect.left + parentRect.width / 2;
    const portalHalfWidth = portalWidth / 2;
    // トリガを中心に左右に十分な余白がある場合
    if (triggerAlignCenter - portalHalfWidth > SPACING &&
        triggerAlignCenter + portalHalfWidth < document.body.clientWidth - SPACING) {
        const insetInlineStart = `${triggerAlignCenter - portalHalfWidth}px`;
        return {
            insetInlineStart,
            insetInlineEnd: undefined,
            maxWidth: `calc(100% - max(${insetInlineStart}, 0px) - ${themes_1.spacing[0.5]})`,
            alignment: 'center',
        };
    }
    // トリガが画面左寄りの場合
    if (triggerAlignCenter <= document.body.clientWidth / 2) {
        const insetInlineStart = `${scrollX + parentRect.left - SPACING}px`;
        return {
            insetInlineStart,
            insetInlineEnd: undefined,
            maxWidth: `calc(100% - max(${insetInlineStart}, 0px) - ${themes_1.spacing[0.5]})`,
            alignment: 'left',
        };
    }
    // トリガが画面右寄りの場合
    const insetInlineEnd = `${document.body.clientWidth - parentRect.right - scrollX - SPACING}px`;
    return {
        insetInlineStart: undefined,
        insetInlineEnd,
        maxWidth: `calc(100% - ${themes_1.spacing[0.5]} - max(${insetInlineEnd}, 0px))`,
        alignment: 'right',
    };
};
//# sourceMappingURL=TooltipPortal.js.map